using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Configuration;

namespace patchdownload {
	/// <summary>
	/// Summary description for Statistic.
	/// </summary>
	public class Statistic : System.Web.UI.Page {
		protected System.Web.UI.WebControls.Table tblLogLines;
		protected System.Web.UI.WebControls.Table tblLogFileWordCount;
		protected System.Web.UI.WebControls.Label lblTotalWordCount;
		protected System.Web.UI.WebControls.Button btnSubmit;
		protected System.Web.UI.WebControls.CheckBox cbSplitByIp;
		protected System.Web.UI.WebControls.Label Label1;
		protected System.Web.UI.WebControls.CheckBox cbCountsOnly;
		protected System.Web.UI.WebControls.Label lblCountsOnly;
		protected System.Web.UI.WebControls.TextBox edtCheckWord;
		protected System.Web.UI.WebControls.DropDownList ddlEndDate;
		protected System.Web.UI.WebControls.Label lblEndDate;
		protected System.Web.UI.WebControls.DropDownList ddlStartDate;
		protected System.Web.UI.WebControls.Label lblStartDate;
		protected System.Web.UI.WebControls.TextBox edtFileName;
		protected System.Web.UI.WebControls.Label lblFileName;
		protected System.Web.UI.WebControls.RadioButtonList rbParseType;
		protected System.Web.UI.WebControls.Label lblError;
	
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e) {
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {    
			this.rbParseType.SelectedIndexChanged += new System.EventHandler(this.rbParseType_SelectedIndexChanged);
			this.btnSubmit.Click += new System.EventHandler(this.btnSubmit_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		Hashtable ipList = new Hashtable();

		string LogPath {
			get { return Server.MapPath(ConfigurationSettings.AppSettings["logPath"]); }
		}

		string DefaultStartDate {
			get { return ConfigurationSettings.AppSettings["defaultStartDate"]; }
		}

		string LogNameFormat {
			get { return ConfigurationSettings.AppSettings["logNameFormat"]; }
		}

		string LogNameDateFormat {
			get { return ConfigurationSettings.AppSettings["logNameDateFormat"]; }
		}

		void AddLogLine(string line) {
			TableRow row = new TableRow();
			tblLogLines.Rows.Add(row);

			TableCell cell = new TableCell();
			row.Cells.Add(cell);
			cell.Width = Unit.Percentage(100);
			cell.Text = line;
		}

		void AddLogFileWordCount(string file, int wordCount) {
			TableRow row = new TableRow();
			tblLogFileWordCount.Rows.Add(row);

			TableCell cell = new TableCell();
			row.Cells.Add(cell);
			cell.Width = Unit.Percentage(20);
			cell.Text = String.Format("{0}:", Path.GetFileName(file));

			cell = new TableCell();
			row.Cells.Add(cell);
			cell.Width = Unit.Percentage(80);
			cell.Text = wordCount.ToString();
		}

		int ProcessFile(string fileName, string checkWord, bool countsOnly) {
			return ProcessFile(fileName, checkWord, countsOnly, cbSplitByIp.Checked);
		}

		int ProcessFile(string fileName, string checkWord, bool countsOnly, bool splitByIp) {
			InitIpCounter();
			int wordCount = 0;

			FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
			StreamReader sr = new StreamReader(fs);
			string s;
			while ((s = sr.ReadLine()) != null) {
				if(s.ToUpper().IndexOf(checkWord.ToUpper()) > -1) {
					if(!countsOnly) 
						AddLogLine(s);
					if(!splitByIp || IsNewIp(GetIp(s))) {
						wordCount++;
					}
				}
			}
			sr.Close();
			fs.Close();
			return wordCount;
		}

		void InitIpCounter() {
			ipList.Clear();
		}

		string GetIp(string line) {
			int ind = line.IndexOf(" ");
			if(ind > -1)
				ind = line.IndexOf(" ", ind + 1);
			if(ind > -1) {
				int indEnd = line.IndexOf(" ", ind + 1);
				if(indEnd > -1)
					return line.Substring(ind + 1, indEnd - ind - 1);
			}
			return String.Empty;
		}

		bool IsNewIp(string ipString) {
			bool result = !ipList.Contains(ipString);
			if(result && !ipString.Equals(String.Empty))
				ipList.Add(ipString, ipString);
			return result;
		}

		int ProcessAllFiles(string checkWord, bool countsOnly) {
			int totalWordCount = 0;
			foreach(string file in Directory.GetFiles(LogPath, "*.*")) {
				int wordCount = ProcessFile(file, checkWord, countsOnly);
				totalWordCount += wordCount;
				AddLogFileWordCount(file, wordCount);
			}
			return totalWordCount;
		}

		int ProcessFilesByDate(string checkWord, DateTime startDate, DateTime endDate) {
			int totalWordCount = 0;

			for(DateTime dt = startDate; dt <= endDate; dt = dt.AddDays(1)) { 
				string file = String.Format(LogNameFormat, dt.ToString(LogNameDateFormat));
				file = String.Format("{0}\\{1}", LogPath, file);

				if(File.Exists(file)) {
					int wordCount = ProcessFile(file, checkWord, true);
					totalWordCount += wordCount;
					AddLogFileWordCount(dt.ToString("dd MMM yyyy"), wordCount);
				}
			} 
			return totalWordCount;
		}

		int ProcessFilesByMonth(string checkWord, DateTime startDate, DateTime endDate) {
			int totalWordCount = 0, monthWordCount = 0;

			endDate = endDate.AddMonths(1);

			int lastMonth = startDate.Month;

			for(DateTime dt = startDate; dt <= endDate; dt = dt.AddDays(1)) { 
				if(lastMonth != dt.Month) {
					AddLogFileWordCount(dt.AddDays(-1).ToString("MMM yyyy"), monthWordCount);
					lastMonth = dt.Month;
					monthWordCount = 0;
				}

				string file = String.Format(LogNameFormat, dt.ToString(LogNameDateFormat));
				file = String.Format("{0}\\{1}", LogPath, file);

				if(File.Exists(file)) {
					int wordCount = ProcessFile(file, checkWord, true);
					monthWordCount += wordCount;
					if(dt < endDate)
						totalWordCount += wordCount;
				}
			} 
			return totalWordCount;
		}

		void UpdateControls() {
			lblFileName.Visible = false;
			edtFileName.Visible = false;

			lblStartDate.Visible = false;
			ddlStartDate.Visible = false;

			lblEndDate.Visible = false;
			ddlEndDate.Visible = false;

			lblCountsOnly.Visible = false;
			cbCountsOnly.Visible = false;

			switch (rbParseType.SelectedItem.Value) {
				case "FileName":
					lblFileName.Visible = true;
					edtFileName.Visible = true;

					lblCountsOnly.Visible = true;
					cbCountsOnly.Visible = true;
					break;
				case "Date":
					lblStartDate.Visible = true;
					ddlStartDate.Visible = true;

					lblEndDate.Visible = true;
					ddlEndDate.Visible = true;

					lblStartDate.Text = "Start Date";
					lblEndDate.Text = "End Date";

					FillDateTimeDdl(ddlStartDate);
					FillDateTimeDdl(ddlEndDate);
					break;
				case "Month":
					lblStartDate.Visible = true;
					ddlStartDate.Visible = true;

					lblEndDate.Visible = true;
					ddlEndDate.Visible = true;

					lblStartDate.Text = "Start Month";
					lblEndDate.Text = "End Month";

					FillMonthDdl(ddlStartDate);
					FillMonthDdl(ddlEndDate);
					break;
			}
		}

		void CheckDataValid() {
			if(edtCheckWord.Text == "")
				throw new Exception("You must specify the checkword value");

			switch (rbParseType.SelectedItem.Value) {
				case "Date":
				case "Month":
					if((ddlStartDate.SelectedItem == null) ||
						(ddlStartDate.SelectedItem.Value == String.Empty) ||
						(ddlEndDate.SelectedItem == null) ||
						(ddlEndDate.SelectedItem.Value == String.Empty))
						throw new Exception("You must specify the date range");
					break;
			}
		}

		void FillDateTimeDdl(DropDownList dateControl) {
			dateControl.Items.Clear();
			dateControl.Items.Add(""); 
			for(DateTime dt = Convert.ToDateTime(DefaultStartDate); dt <= DateTime.Today; dt = dt.AddDays(1)) { 
				dateControl.Items.Add(dt.ToString("dd MMM yyyy")); 
			} 
		}

		void FillMonthDdl(DropDownList dateControl) {
			dateControl.Items.Clear();
			dateControl.Items.Add(""); 
			for(DateTime dt = Convert.ToDateTime(DefaultStartDate); dt <= DateTime.Today; dt = dt.AddMonths(1)) { 
				dateControl.Items.Add(dt.ToString("MMM yyyy")); 
			} 
		}

		void Page_Load(object sender, System.EventArgs e) {
			lblError.Text = "";
			if(!IsPostBack) {
				rbParseType.SelectedIndex = 0;
				cbCountsOnly.Checked = true;
				cbSplitByIp.Checked = true;
				edtFileName.Text = "ex040529.log"; //String.Format(LogNameFormat, DateTime.Today.ToString(LogNameDateFormat));
				UpdateControls();
			}
		}

		void btnSubmit_Click(object sender, System.EventArgs e) {
			try {
				CheckDataValid();

				tblLogLines.Visible = !cbCountsOnly.Checked;

				switch (rbParseType.SelectedItem.Value) {
					case "FileName":
						if(edtFileName.Text != "") 
							lblTotalWordCount.Text = ProcessFile(String.Format("{0}\\{1}", LogPath, edtFileName.Text),
								edtCheckWord.Text, cbCountsOnly.Checked).ToString();
						else
							lblTotalWordCount.Text = ProcessAllFiles(edtCheckWord.Text, cbCountsOnly.Checked).ToString();
						break;
					case "Date":
						lblTotalWordCount.Text = ProcessFilesByDate(edtCheckWord.Text,
							Convert.ToDateTime(ddlStartDate.SelectedItem.Value), Convert.ToDateTime(ddlEndDate.SelectedItem.Value)).ToString();
						break;
					case "Month":
						lblTotalWordCount.Text = ProcessFilesByMonth(edtCheckWord.Text,
							Convert.ToDateTime("01 " + ddlStartDate.SelectedItem.Value),
							Convert.ToDateTime("01 " + ddlEndDate.SelectedItem.Value)).ToString();
						break;
				}
			}
			catch (Exception ex) {
				lblError.Text = ex.Message;
			}
		}

		void rbParseType_SelectedIndexChanged(object sender, System.EventArgs e) {
			UpdateControls();
		}
	}
}
